# Changelog

All notable changes to the Rust Crate Pipeline project.

## [1.2.4] - 2025-06-18

### 🐛 Critical Logging Fix

#### ✨ Fixed
- **Critical Logging Issue**: Fixed 0-byte log file problem caused by conflicting `logging.basicConfig()` calls
- **Enhanced File Logging**: Improved logging setup with proper handler management and UTF-8 encoding
- **Better Error Tracking**: Now properly logs all processing steps, errors, and skipped crates to file
- **Console + File Output**: Maintains both console output and detailed file logging

#### 🔧 Improved
- **Logging Conflicts**: Resolved production config vs main config logging conflicts
- **File Handler**: Added proper error handling for log file creation
- **Encoding Issues**: Fixed Unicode handling in log files
- **Debug Information**: Always captures DEBUG+ level info to log files while respecting console log level

#### 📊 Monitoring
- **Better Tracking**: Now you can properly see which crates were skipped and why
- **Detailed Logs**: Each processing step is properly logged with timestamps
- **Error Analysis**: Failed crates and reasons are now captured in log files

## [1.2.3] - 2025-06-18

### 🚀 L4 GPU Optimization Release

#### ✨ Added
- **L4 GPU-Optimized Model Loading**: Configured for GCP g2-standard-4 with L4 GPU (24GB VRAM)
  - Larger context window (`n_ctx=4096`) leveraging L4's memory capacity
  - Aggressive GPU layer loading (`n_gpu_layers=-1`) for maximum performance
  - Optimized batch size (`n_batch=1024`) for L4 throughput
  - CPU thread optimization (`n_threads=4`) matching g2-standard-4's 4 vCPUs
  - Enhanced memory management with `use_mmap=True` and `use_mlock=True`
  - Flash attention support (`flash_attn=True`) for faster computation
  - RoPE scaling configuration for extended context processing

- **Batch Processing System**: New `batch_process_prompts()` method for GPU utilization
  - Processes multiple prompts simultaneously (batch_size=4 optimized for L4)
  - Thermal management with inter-batch delays
  - Enhanced sampling parameters (`top_p=0.95`, `repeat_penalty=1.1`)
  - Robust error handling for batch operations

- **Smart Context Management**: New `smart_context_management()` method
  - Prefix cache optimization for better performance
  - Intelligent context reuse prioritizing recent history
  - Dynamic token allocation up to 4000 tokens
  - Smart truncation maintaining context relevance

#### 🔧 Changed
- **Performance Improvements**: Expected 3-4x faster inference on L4 GPU vs CPU-only
- **Memory Optimization**: Better utilization of L4's 24GB VRAM capacity
- **Quality Enhancements**: Improved sampling and context management

#### 📈 Performance
- Significant throughput improvements on GCP g2-standard-4 instances
- Reduced per-prompt processing overhead through batching
- Enhanced cache efficiency with smart context reuse

## [1.2.1] - 2025-06-18

### 🔒 Security & Performance Update

#### ✨ Added
- **Enhanced Docker security** with specific base image versioning (`python:3.11.9-slim-bookworm`)
- **Improved AI validation retry logic** with 4 attempts instead of 2 for better success rates
- **More generous temperature scaling** (20% increases vs 10%) for better AI response variety
- **Extended wait times** between AI retries (2-5s vs 1-1.5s) for better model performance
- **Enhanced health checks** with proper functionality testing
- **Security environment variables** (`PYTHONNOUSERSITE`, `PYTHONHASHSEED`)

#### 🔧 Changed
- **Validation warnings reduced to debug level** - much cleaner console output during inference
- **Improved parameter allocation** for AI tasks (increased token limits and better temperatures)
- **Better prompt simplification strategy** - only simplifies on later attempts
- **Enhanced Docker metadata** with OCI labels and security updates

#### 🐛 Fixed
- **AI validation timeout issues** by providing more time and attempts for complex tasks
- **Docker vulnerability exposure** through system security updates and specific versioning
- **Inconsistent AI response generation** through improved retry logic and parameter variety

#### 📈 Performance
- **Significantly reduced "Final validation attempt failed" warnings**
- **Higher AI task success rates** through better retry strategies
- **More reliable Docker container health checks**

## [1.2.0] - 2025-06-18

### 🚀 Major Release - Production Ready

**This is a significant release that transforms the project into a production-ready, PyPI-published package.**

#### ✨ Added
- **Complete PyPI package structure** with proper metadata and entry points
- **Unified comprehensive README.md** consolidating all documentation
- **Production-optimized configurations** with reduced runtime warnings
- **Enhanced GitHub token integration** with automatic detection and setup
- **Docker deployment ready** with optimized containers and compose files
- **Comprehensive CLI interface** with help system and examples
- **Professional package metadata** with badges, descriptions, and links

#### 🔧 Changed
- **Moved all source code** into proper `rust_crate_pipeline/` package structure
- **Consolidated documentation** from multiple files into single unified README
- **Optimized import structure** for better package organization
- **Enhanced error handling** with graceful degradation
- **Improved logging** with appropriate levels for production

#### 🗑️ Removed
- **All non-essential files** including test scripts and development artifacts
- **Redundant documentation** files (PROJECT_STRUCTURE.md, various guides)
- **Development-only modules** (optimizations.py, docker_optimizations.py)
- **Windows-specific scripts** focusing on Linux production deployment
- **Cache files and build artifacts** for clean distribution

#### 🛡️ Security
- **Enhanced GitHub token handling** with secure storage and validation
- **Production environment configurations** with appropriate access controls

#### 📦 Distribution
- **Published to PyPI** as `rust-crate-pipeline`
- **Docker Hub ready** with multi-stage builds
- **Comprehensive installation options** (PyPI, source, development)

#### 🔄 Migration Notes
- Package is now installed via `pip install rust-crate-pipeline`
- All functionality preserved with enhanced reliability
- New unified documentation provides complete usage guide
- GitHub token setup simplified with interactive script

---

## [1.1.1] - 2025-06-18

### 🚀 Major Features Added

#### GitHub Token Integration
- **Automatic token detection**: Pipeline now checks for GitHub token on startup
- **Interactive setup prompts**: Guides users through token configuration if missing
- **Token validation**: Verifies token works with GitHub API before processing
- **Setup scripts**: Added `setup_github_token.sh` for easy Linux configuration
- **Verification tool**: `check_github_token.py` for comprehensive token testing

#### Production Optimizations
- **Production mode**: `PRODUCTION=true` environment variable for optimized settings
- **Reduced warnings**: GitHub API rate limit warnings minimized (100→500 threshold)
- **Smart retries**: LLM validation retries reduced from 3→2 attempts
- **Enhanced logging**: Appropriate log levels (DEBUG/INFO/WARNING) based on context
- **Production launcher**: `run_production.py` script with optimized defaults

#### Docker & Deployment Ready
- **Production Dockerfile**: Optimized container configuration
- **Docker Compose**: Ready-to-use orchestration with environment variables
- **SSH deployment**: Background execution with logging and monitoring
- **Systemd service**: Linux service configuration for persistent execution
- **Environment detection**: Automatic production vs development mode switching

### 🔧 Technical Improvements

#### Package Structure
- **PyPI ready**: Complete package configuration with `pyproject.toml`
- **Proper imports**: Fixed all relative imports and module structure
- **Version management**: Centralized version handling with `version.py`
- **Dependencies**: Updated `requirements.txt` with all necessary packages
- **Build system**: Added `setup.py` and `MANIFEST.in` for distribution

#### Code Quality
- **Error handling**: Enhanced exception handling throughout pipeline
- **Type hints**: Improved type annotations where possible
- **Configuration**: Centralized settings with production overrides
- **Logging**: Structured logging with configurable levels
- **Testing**: Added integration tests and verification scripts

#### Performance & Reliability
- **Smart rate limiting**: Proactive GitHub API management
- **Caching optimization**: Enhanced request caching for better performance
- **Memory management**: Optimized batch processing and resource usage
- **Fallback strategies**: Graceful degradation when services unavailable
- **Progress tracking**: Improved checkpointing and progress reporting

### 📦 New Files Added

#### Configuration & Setup
- `rust_crate_pipeline/production_config.py` - Production settings
- `rust_crate_pipeline/optimizations.py` - Performance optimization utilities
- `rust_crate_pipeline/github_token_checker.py` - Token validation module
- `setup_github_token.sh` - Interactive Linux token setup
- `check_github_token.py` - Comprehensive token verification

#### Docker & Deployment
- `Dockerfile` - Production container configuration
- `docker-compose.yml` - Orchestration configuration
- `.dockerignore` - Container build optimization
- `docker-entrypoint.sh` - Container startup script

#### Testing & Validation
- `test_optimizations.py` - Test optimization modules
- `test_token_integration.py` - Test GitHub token integration
- `run_production.py` - Production launcher script

#### Documentation
- Consolidated `README.md` - Comprehensive user guide
- `CHANGELOG.md` - Version history and improvements

### 🐛 Bug Fixes
- **Import errors**: Fixed `relativedelta` import by switching to `python-dateutil`
- **Module structure**: Resolved package import issues
- **Configuration**: Fixed config file loading and environment variable handling
- **Docker builds**: Resolved container build and runtime issues
- **Error reporting**: Improved error messages and user guidance

### 🔄 Breaking Changes
- **Package structure**: Moved all source files to `rust_crate_pipeline/` directory
- **Dependencies**: Added `python-dateutil` as required dependency
- **Configuration**: Some config options moved to production-specific settings

### 📈 Performance Improvements
- **API efficiency**: Reduced redundant GitHub API calls
- **Memory usage**: Optimized batch processing and caching
- **Startup time**: Faster initialization with lazy loading
- **Error recovery**: Better handling of transient failures
- **Resource monitoring**: Enhanced disk space and memory tracking

### 🎯 User Experience
- **Clearer messages**: Improved user feedback and error messages
- **Guided setup**: Step-by-step token configuration assistance  
- **Production readiness**: One-command deployment for production use
- **Monitoring tools**: Built-in status checking and validation
- **Documentation**: Comprehensive guides for all use cases

## [1.0.0] - Previous Release

### Initial Features
- Basic crate metadata collection
- AI-powered enrichment
- Source code analysis
- Dependency mapping
- Command-line interface

---

**Migration Guide**: To upgrade from 1.1.1 to 1.2.0, simply install the new version via `pip install --upgrade rust-crate-pipeline`. All functionality is preserved with enhanced reliability and the new unified documentation provides a complete usage guide.
